//version 17.08.2017: in Hassanat metric formula simplified (deleted terms giving sum equal to zero).
//	State of experimental metrics: Haha uses Na2, Ha22c uses Na2.
//version 17.02.2017: added contrl function.
//version 29.12.2016> added procedures for histograms and double histograms.
//version 19.08.2016: added Huber's and Tucker's regression.
//version 23.01.2016: added Fibonacci number, Optimization with Fibonacci section method, linear regression, saginess evaluation
//	Weiersterass and Cayley-Klien-Hilbert metrics repared. 
//	Function arccosh included.
//version 26.11.2015: added metrics, covariance matrix computation, distance function
#include        <math.h>
#include        <stdlib.h>
#include        <stdio.h>
#include        <string.h>
#include        <time.h>  // Needed for void MyRandomize() only.

typedef struct {//structure for compare procedure for sorting
		double D;
		long I;
		long V;
		double der;
} TDist;

void printmatf (double* A, long dim){
	long i,j;
	for(i=0;i<10;i++){
		for(j=0;j<dim;j++) printf("%10lg  ",A[i*dim+j]);
		printf("\n");
	}
}

int TDistCompare(const void *arg1, const void *arg2 ){
   //Compare both parameters for quicksort
   //qsort(array, NumOfElements, sizeof(TDist), TDistCompare );
        //printf("myfu23:\n");
	TDist x,y;
        x=*(TDist*)arg1;//je hodnota prvniho argumentu
        y=*(TDist*)arg2;
        if(x.D <y.D) return -1;
        if(fabs(x.D-y.D)<1.0e-33) return  0;
        else return  1;
}

/* nonexisting operations */

double arccosh(double x){//arc cos hyperbolic (acosh, arcosh, argcosh) for x>=1, else returns -1 as an error message.
	if (x<1) return -1;//as an error value
	return log(x+sqrt(x*x-1));
}

double interpolFast(double x, double *xrow, double *yrow, long rows, long from, long interpolType){
    //Data musi byt serazena ve sloupci pro x vzestupne a nemaji byt dve hodnoty stejne, ale kdy jsou, bere se pak prmr odpovidajicich yrow
    //promenna from je obvykle 0, coz znamena prohledavani od zacatku; jinak se prohledava od xrow[from].
	//Typy interpolace: 1=linear, 2=Akima (not used now), ...
    long i;
	double xd,yd,xh,yh;
	//"from" out of range
    if(from < 1 || from > rows) from = 0;
    
    //x out of range
    if(x <= xrow[0]) {
        from = 0;
        return yrow[0];
	}
    if(x >= xrow[rows-1]) {
        from = 0;
        return yrow[rows-1];
    }
    //Linear
    //regular
    xd=0.0;
    for(i = from; i<rows;i++) {
        xd = xrow[i];
        xh = xrow[i+1];
        yd = yrow[i];
        yh = yrow[i+1];
        if(x == xd && xd == xh) {
            from = i;
            return (yd + yh) / 2;
        }
        if(x >= xd && x < xh){
            from = i;
            return yd + (x - xd) * (yh - yd) / (xh - xd);
		}
	}
    from = 0;
	return xd;
}

double Na2(double x){
        return x*x;
}

double Na(double x, double n){
        double y, z;
        //long i;
        y=fabs(x);
        if(fabs(n-1)<1.0e-33) return y;
        if(log10(y)*n>230) return 1.0e100;
        z=1.0;
        //for (i=0;i<n;i++) z=z*y;
        z=exp(n*log(y));
        return z;
}

double sign(double x) {
  return (x>=0?1:-1);
}

double frac(double x){
  double y;
  y=x-floor(x);
  return y;
}


double WeibullCDF(double x,double lambda,double k){//p1=lambda=beta, p2=k=alpha p3 undefined   
	return 1-exp(-Na(x/lambda,k));
}

double ParetoCDF(double x,double xm,double k){//p1=xm,  p2=k  p3   undefined             
	return 1-Na(xm/x,k);
}

double ParetoCDFsh2zero(double x,double xm,double k){//p1=xm,  p2=k  p3   undefined             
	return 1-Na(xm/(1+x),k);
}





//***************************************************************************
// >> C O N V E N C T I O N > functions give the value of variable, not a probability 
// for indexes 0 ... n; total n+1 steps are supoposed, i.e.
// step i corresponds to the probability equal to (i+1)/(n+1) 
// 0 corresponds to 1/(n+1) probability, and n to n/(n+1) probability
//**************************************************************************
//x=ADGcrit(NoRows, pist, psoll); 
double ADGcrit(long n,double *x, double *ff0){//Anderson-Darling General criterion
//                        ^^experiment    ^^^teoretick, kdyz srovnvm s teoretickm
//HERE  x and ff0 are PROBABILITIES for for equidistant data !!!!!!!!!!!!!!!
//                = Astar^2 in the algorithm from Wiki !!not by Scholz-Stephens
//                  dle  Annis-12-Anderson-Darling.PDF
        long i;
        double A2,xn;
	A2=0;//dle Anderson-Darling-two-sample.pdf
        for(i=0;i<n;i++){
            A2+=Na2(x[i]-ff0[i])/(ff0[i]*(1-ff0[i]));
	}
	xn=(double)n;
        return A2*(1+4/xn-25/(xn*xn));
}

double ADGprob(double tkn){//Anderson-Darling General distribution
//  = procento nejistoty kdyz zamtnu shodu rozdelen
// Input criterion tkn, output significance level (percentage)
// aproximated to be equal or a little bit worse (larger)
	double ast,p;
	ast=tkn; p=0.0;
	if(ast>13) ast=13;
	if (0.600 < ast /* && ast < 13 */) p = exp(1.2937 - 5.709*ast + 0.0186*ast*ast);
	if (0.340 < ast && ast < 0.600) p = exp(0.9177 - 4.279*ast - 1.38*ast*ast);
	if (0.200 < ast && ast < 0.340) p = 1 - exp(-8.318 + 42.796*ast - 59.938*ast*ast);
	if (ast < 0.200) p = 1 - exp(-13.436 + 101.14*ast - 223.73*ast*ast);
	if(p>1.0) p=1.0;
	if(p<0.0) p=0.0;
	return p;
/* Description
If I know AD (or A-Squared), how can I calculate the Anderson-Darling Normality Test p-value?
 Solution:
Suppose asq = AD, and n = number of observations.
Let ast = asq*(1 + 0.75/n + 2.25/(n*n)).
-> If 0.600 < ast < 13, then p = exp(1.2937 - 5.709*ast + 0.0186*ast*ast).
-> If 0.340 < ast < 0.600, then p = exp(0.9177 - 4.279*ast - 1.38*ast*ast).
-> If 0.200 < ast < 0.340, then p = 1 - exp(-8.318 + 42.796*ast - 59.938*ast*ast).
-> If ast < 0.200, then p = 1 - exp(-13.436 + 101.14*ast - 223.73*ast*ast).
Reference: R.B. D'Agostino and M.A. Stephens, Eds. (1986). Goodness-of-Fit Techniques, Marcel Dekker.
*/
}

double ADGprobM(double tkn){//Anderson-Darling General distributioni; my approximation
// Input criterion tkn, output significance level (percentage)
//   aproximated to be equal or a little bit worse (larger)
        double p;
        p=0.09*Na(tkn+0.35,6.3);
        if(p>1.0) p=1.0;
	return p;
}


double CvMprob50(double CSvM, long count) {
	//Cramer von Mises test of distributions
	//area = test parameter, area between distrib.functions
	//count = the second parameter of test, the number of empirical points
	//values are valid for exactly 50 empirical points counts; practical differences for the other "counts" are small.
	//double x[18], p[18];
	double x[18]={0, 0.02512, 0.03068, 0.03690, 0.04636, 0.05462, 0.06258, 0.07062, 0.11924, 0.20958, 0.24132, 0.28396, 
				  0.346822, 0.45986, 0.57754, 0.73728, 1.14507, 2};
	double p[18]={0, 0.01, 0.025, 0.05, 0.1, 0.15, 0.2, 0.25, 0.5, 0.75, 0.8, 0.85, 
				  0.9, 0.95, 0.975, 0.99, 0.999, 1 };
	//double interpolFast(double x, double *xrow, double *yrow, long rows, long from, long interpolType)
	return interpolFast(CSvM, x, p, 18, 0, 2);//[Akima]
}

double CvMprob(double tkn){ // Cramer von Mises General distribution MY approximation
//! If TkN exceeds the given point then 
//!   reject Ho at significance level "p". 
//! Input criterion tkn, output significance level (percentage)
//!   aproximated to be equal or a little bit worse (larger)
        return 1-Na(0.989-1.094*tkn,10);
}

double CvMcrit(long n, long m, double* fx, double* ff){ // Cramer von Mises General criterion MMMM
        long j,i;
        double A2, z,ii,jj,jff;
        A2=1/(12*n); j=0; z=0.0;
       	for(i=0; i<n; i++){
		ii=(double)i;
		for(j=0;j<m;j++) {
			jj=j;
			if(ff[j]<=fx[i] && fx[i]<ff[j+1]){
				jff=jj+(1.0)*(fx[i]-ff[j])/(ff[j+1]-ff[j]);
				z=fabs((ii+1)/(n+1)-(jff+1)/(m+1));
				break;
			}
		}
		A2+=z*z;
	}
        return A2/n;
}


double KSprob(double alam){ //!Kolmogorov-Smirnov probability function
	long j;
        double EPS1,EPS2,KSprob;
        EPS1=0.001; 
	EPS2=1.e-8;
        double a2,fac,term,termbf;
        a2=-2.0*Na2(alam);
        fac=2.0;
        KSprob=0.0;
        termbf=0.0;// !Previous term in sum.
        for(j=1; j<=100;j++){
            term=fac*exp(a2*j*j);
            KSprob=KSprob+term;
            if(fabs(term)<=EPS1*termbf || fabs(term)<=EPS2*KSprob) return KSprob;
            fac=-fac; // !Alternating signs in sum.
            termbf=fabs(term);
        }
        return 1.0; //Get here only by failing to converge.
}


double KSdif(long n, long m,  double* fx, double* ff){ // different number of samples n and m is supposed 
	//fx[n], ff[m],   n>m
        double z,ii,jj,jff;
        long i,j;
        z=0.0;
       	for(i=0; i<n-1; i++){
//printf("%li   %lg\n",i,fx[i]);
		if(fabs(fx[i]-fx[i+1])>1.0e-4){
			ii=(double)i;
			for(j=0;j<m-1;j++) {
				if(fabs(ff[j]-ff[j+1])>1.0e-4){
					jj=(double)j;
//printf("j i    ffj fxi ffj+1: %li %li     %lg %lg %lg   ",j,i,ff[j],fx[i], ff[j+1]);
					if(fx[i]<ff[j]) break;
					if(ff[j]<=fx[i] && fx[i]<ff[j+1]){
						jff=jj+(1.0)*(fx[i]-ff[j])/(ff[j+1]-ff[j]);
						if(fabs((ii+1)/(n+1)-(jff+1)/(m+1)) > z) 
							z=fabs((ii+1)/(n+1)-(jff+1)/(m+1));
//printf("z = %lg\n",z); 
					}
				}
//getchar();
			}
		}
	}
        return z;  //0<=z<=1  !!!
}


double KScrit(long n, double ksdif){ //!ksone(data,n,func,d,prob)!onesided KS criterion
        double en;
        en=(double)n;
        en=sqrt(en);
        return (en+0.12+0.11/en)*ksdif;
}

//******************************************************!*********************
// >> C O N V E N C T I O N > functions give the value of variable, not a probability 
// for indexes 0 ... n; total n+1 steps are supoposed, i.e.
// step i corresponds to the probability equal to (i+1)/(n+1) 
// 0 corresponds to 1/(n+1) probability, and n to n/(n+1) probability
//**************************************************************************



double Gam(long a){     //see http://www.xycoon.com/beta_mode.htm
        long i;
        double x;
        x=1;
        for(i=1;i<=a;i++) x=x*((double)rand()/RAND_MAX);
        if(x<1.0e-30) x=(double)rand()/RAND_MAX;
        return log(x);
}

double BetaDist(long a, long b){     //see http://www.xycoon.com/beta_mode.htm
        double GamA;
        GamA=Gam(a);
        return GamA/(GamA+Gam(b));
}

double NormDist01(){    //see http://www.xycoon.com/nor_random.htm
        double x;
        long i;
        x=0;
        for(i=0;i<12;i++) x+=(double)rand()/RAND_MAX;
        return x-6;
}

double LinRegXY(double *X, double *Y, double* slope, double *C, 
		long N, double *residu){
	//A standard linear regression of function Y=f(X) giving slope and constant C.
	//returns sigma, i.e. sqrt of the mean residuum squared.
	double Sri2, Sri, Si, Siri, ri, Iapr, sumresidu2;
	long i;
	Sri2=0; Sri =0; Si  =0;  Siri=0;
	for(i=0;i<N;i++) {
		ri=X[i]; 
		Sri2=Sri2+ri*ri;
		Sri =Sri+ri;
		Si  =Si+Y[i];
		Siri=Siri+Y[i]*ri;
	}//thus we have data for regression equation; N is the last point used
	slope[0]=(N*Siri-Sri*Si)/(N*Sri2-Sri*Sri);
	C[0]=  (Si*Sri2-Sri*Siri)/(N*Sri2-Sri*Sri);//regression constant 
	//RESIDUALS evaluation
	sumresidu2=0;
			for(i=0;i<N;i++) {
		ri=X[i]; 
		Iapr=C[0]+slope[0]*ri;//linear approximation
		residu[i]=Y[i]-Iapr;//residuum 
		sumresidu2+=Na2(residu[i]);
	}
	return sqrt(sumresidu2/N);
}


double WLReg2(double *X, double *Y, double *W, double* beta, 
		long N, double *residu){
	//A standard linear regression of function Y=f(X) giving slope and constant C.
	//returns sigma, i.e. sqrt of the mean residuum squared.
	double x,y,w, Sw, Swx, Swx2, Swy, Swxy, determ, iapr, sumresidu2;
	long i;
	Sw=0; Swx =0; Swx2 =0;  Swy=0; Swxy=0;
	for(i=0;i<N;i++) {
		x=X[i];
		y=Y[i];
		w=W[i]; 
		Sw+=w;
		Swx+=w*x;
		Swx2+=w*x*x;
		Swy+=w*y;
		Swxy+=w*x*y;
//printf("%lg %lg %lg\n",x,y,w);	

	}//thus we have data for regression equation; N is the last point used
	determ=Sw*Swx2-Swx*Swx;
	if(fabs(determ)<1.0e-100){
		beta[0]=0.0;
		beta[1]=0.0;
		return 1.0e33;
	}
	beta[0]=(Swx2*Swy-Swx*Swxy)/determ;//constant
	beta[1]=(-Swx*Swy+Sw*Swxy)/determ;//slope 
//printf("WLReg2: %lg %lg %lg\n",determ,beta[0],beta[1]);	
//getchar();
	//RESIDUALS evaluation
	sumresidu2=0;
	for(i=0;i<N;i++) {
		x=X[i]; 
		iapr=beta[0]+beta[1]*x;//linear approximation
		residu[i]=Y[i]-iapr;//residuum 
		sumresidu2+=Na2(residu[i]);
	}
	return sqrt(sumresidu2/N);
}

double HuPsi(double x){//Huber's psi function
	double c, mi;
	c=1.345;
	mi=c<x?c:x;
	return (-c>mi?-c:mi);
}

double TuPsi(double x){//Tucker's psi function
	double c, rc;
	c=4.685;
	rc=x/c;
	if(x<=c) return x*Na2(1-Na2(rc));
	else return 0;
}

double TuHuLReg2(double *X, double *Y, double* beta, 
		long N, double *residu, double eps, long TuHu){
	//Huber (TuHu=1) or Tucker (TuHu=2) robust regression
	// (else standard leaast squares regression).
	double sigma, betaold, alfa;
	long i,j;
	double* W = new double [N];
	for(i=0;i<N;i++) W[i]=1;
	beta[0]=1.0e33;
	alfa=0.1;
	for(j=0;j<10000;j++){//max number of iterations
		betaold=beta[0];
		sigma=WLReg2(X,Y,W,beta,N,residu);
		if(fabs(beta[0]-betaold)<eps || TuHu<1 || TuHu>2) return sigma;
//	printf("new weights beta0=%lg betaold=%lg TuHu=%li eps=%lg\n",beta[0],betaold,TuHu,eps);
//	getchar();
		for (i=0;i<N;i++){
			if(TuHu==1) W[i]=HuPsi(residu[i]/sigma)/(residu[i]/sigma);
			if(TuHu==2) W[i]=TuPsi(residu[i]/sigma)/(residu[i]/sigma);
		}
	}
	return sigma;
}


long XXX200 (double *a, double *b, long N, double EPSILO, double *DET){
/* funkce vypocita reseni soustavy linearnich rovnic a
         vraci 1 pro regularni vysledek, 0 jinak.
                         PIVOTY VYBIRA POSTUPNE NA HLAVNI UHLOPRICCE
         A = POLE MATICE ULOZENE PO RADCICH; ZMENI SE NA JEDNOTKOVOU MATICI
         B = VEKTOR PRAVYCH STRAN, KTERY SE ZMENI NA VEKTOR RESENI
         N = STUPEN MATICE
         EPSILO = ZADANA MINIMALNI HODNOTA PIVOTU
         *DET = (POINTER NA) DETERMINAT  (pi voln pst &DET)          */
        long i,j,k;
        double x;
        if(N<=0)return(1);/* nesmysna dimense */
        if(N<2){          /* matice je typu 1x1 */
                if(fabs(a[0])>=EPSILO) {a[0]=1./a[0];   return(1);}
                else {return(0);}
        }
        *DET=1.0;                               /* normalni matice */
        for(i=0;i<N;i++){ /* hlavni krokovani po uhlopricce */
                if(fabs(a[N*i+i])<EPSILO)
                  return(0); /* OSETRENI PIVOTU */
                *DET=*DET*a[N*i+i];
                x=1/a[N*i+i];           /* deleni i-teho radku pivotem */
                for(k=i;k<N;k++) a[N*i+k]=a[N*i+k]*x;/*for(k=0;...misto n^2 jenn(n-1)/2 nasobeni*/
                b[i]=b[i]*x;
                for(j=0;j<N;j++){/* krokovani po radcich, vynechame i-ty */
                        if(j!=i){
                                x=a[N*j+i]; /* prvek pro vynas. i-teho radku a jeho odect. od radku j */
                                for(k=i;k<N;k++) a[N*j+k]=a[N*j+k]-x*a[N*i+k];/*for(k=0;...misto n^3 jen n^2(n-1)/2 nasobeni*/
                                b[j]=b[j]-x*b[i];
                        }
                }
        }
        return(1);
}


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	function [Z,D] = ainvl(A,tol);	 norm(inv(A)-Z*Z')-->0
void ainvl(double *z,double *d,double *a,long n,double tol){
	//
	// left-looking ainv decomposition
	//
	// purpose:
	//   Computes the factorization 
	//         ZD^{-1}Z^T  \approx inv(A) ??????????????
	//   of the symmetric and positive definite matrix A.
	//   simple dense code,
	//   where D is diagonal matrix of squares of eigenvalues
	//
	// input:
	//   A:    matrix to be factored;
	//   tol:  drop tolerance for elements in U and V factors;
	//
	// output:
	//   Z,D:  AINV factors.
	// returns 1 if OK else 0 if pivot is lesser than 1.0e-30.
	//
	// history:
	//   Matlab code by Mirek Tuma, 2003.
	//   FORTRAN code by Marcel Jirina, 2014. Needs subroutines myrow, mycol, scas.
	//   c++ code by Marcel Jirina, 2014. No own externals.
	//
	// 	basic initializations
	//
	long i,j,k,l;
	double pi,pj,pp,ajnk,zknj,xinvpp;
	double* dd  = new double [n];
	//
	// unit matrices
	pp=0.0;
	for(i=0;i<n;i++){
		for(j=0;j<n;j++) z[i*n+j]=0;
		d[i]=z[i*n+i]=1;
	}
	//
	// the loop
	//
	for(i=0;i<n;i++){
		//
		// pivoting
		//
		for(j=0;j<i;j++){
			//
      		// find the multipliers
      		//
			pi=pj=pp=0;
			for(k=0;k<n;k++){
					pi+=(ajnk=a[j*n+k])*z[k*n+i];
					pj+=a[i*n+k]*(zknj=z[k*n+j]);
					pp+=ajnk*zknj;
			}
      		//
      		// update the remaining columns
	      	//
			if(fabs(pp)<1.0e-30) goto nic;
			xinvpp=1/pp;
			//change the i-th column of z:
      		//Z(:,i) = Z(:,i) - Z(:,j)*inv(pp)*pj;
			for(k=0;k<n;k++){  //do k=1,n
				z[k*n+i]=z[k*n+i]-z[k*n+j]*xinvpp*pj;
			}
      		//
      		// dropping
      		//
			for(k=0;k<n;k++){  
        		if(fabs(z[k*n+i]) < tol) z[k*n+i]=0.0;
      		}
    	}
		pp=0;
		for(k=0;k<n;k++) pp+=a[i*n+k]*z[k*n+i];
		d[i*n+i]=pp;               //D(i,i) = pp; 
		dd[i]=sqrt(1/pp);
   		// 
  	}
	//
	//Z=Z*sqrt(inv(D)):	D is diagonal
	//
	for(k=0;k<n;k++){
		for(l=0;l<n;l++){
			z[k*n+l]=z[k*n+l]*dd[l];
		}
	} 
nic:;//case of singular matrix A --> z and d are then unit matrices
	if(fabs(pp)<1.0e-30){
		for(i=0;i<n;i++){
			for(j=0;j<n;j++) z[i*n+j]=0;
			d[i]=z[i*n+i]=1;
		}
	}
	//return 1;
	//  norm(inv(A)-Z*Z')
}   // of ainvl   ////////////////////////////////////////////////////////////



double der3(double* x, double* y, long i, long samples){
	//the first derivative at point (x0,y0) using left and right neighbors	
	if(i<1 || i>samples-3) return 0.0;//if out of range	
	return (y[i+1]-y[i-1])/(x[i+1]-x[i-1]);
}

double der7h(double* x, double* y, long i, long samples){
	//the fist derivative at point (x0, y0) by Holoborodko's 7-point formula exact to x^4.
	//Needs total 7 points symmetrically around point (x0, y0).
	if(i<3 || i>samples-3 || fabs((39*(y[i+1]-y[i-1])+12*(y[i+2]-y[i-2])-5*(y[i+3]-y[i-3]))) 
		>1.0e33 || fabs(48*(x[i+1]-x[i-1]))<1.0e-33) return 0.0;//if out of range
	return (39*(y[i+1]-y[i-1])+12*(y[i+2]-y[i-2])-5*(y[i+3]-y[i-3]))/(48*(x[i+1]-x[i-1]));
}  


double dder5(double* x, double* y, long i, long samples){
	//the SECOND derivative at point (x0, y0); 
	//needs total 5 points symmetrically around (x0,y0). 
	double xx[3], z[3];
	long j;
	if(i<2 || i>samples-2) return 0.0;//if out of range
	for(j=0;j<3;j++) {
		z[j]=der3(x, y, i+j-1, samples);	
		xx[j]=x[i+j-1];
	}
	return der3(xx, z, 1, samples);
}


double dder13h(double* x, double* y, long i, long samples){
	//the SECOND derivative at point (xi, yi) by first derivatives 
	//by Holoborodko's 7-point formula; exact to x^4. 
	//Needs total 13 points symmetrically around point (xi, yi).
	double z[13], xx[13], rr;
	long j;	
	if(i<6 || i>samples-6) return 0.0;
	for(j=0;j<13;j++) {
		z[j]=der7h(x, y, i+j-6, samples);	
		xx[j]=x[i+j-6];
	}
	rr=der7h(xx, z, 6, samples);
	return rr;
}


  

double mahala(double *u, double *v, double *z, long n){
//	 v'*inv(a)*u;
	double vysledek;
	double* dd  = new double [n];
	double* ls  = new double [n];
	long i,j;
	for(i=0;i<n;i++) dd[i]=u[i]-v[i];
	for(i=0;i<n;i++) {
		ls[i]=0;
		for(j=0;j<n;j++) ls[i]+=z[i*n+j]*dd[j];
	}
	//scalar product
	vysledek=0;
	for(i=0;i<n;i++) vysledek+=ls[i]*ls[i];
	delete ls;
	delete dd;
	return sqrt(vysledek);
} // end of mahala //////////////////////////////////////////////////////////////





char * MYstrncpy(char * s1, char * s2, size_t nn){
        char * k;long m, n;
        m=(long)strlen(s2);
        n=(long)nn;
        n=m>n?n:m;
        k=strncpy(s1, s2, n);
        s1[n]=(char)NULL;
        return k;
};

long MyStrCmp(char *a, char *b){     //Porovnani retezcu
        long i, x, y;
        x=strlen(a);
        y=strlen(b);
        if(x!=y) return 0;
        for(i=0;i<x && i<y;i++){
                if(a[i]!=b[i]) return 0;
        }
        return 1;
}

void MYstrcpy(char *a, char *b, long shiftA, long shiftB, long n){ //kopirovani retezce b do a
        long i;
        for(i=0;i<n;i++) {
                if(b[i+shiftB]==char(NULL)) break;
                a[i+shiftA]=b[i+shiftB];
        }
        a[i+shiftA]=char(NULL);
        return;
}



void MyRandomize(){ // needs #include <time.h>
    long t, i;//time_t t;

    t = time(NULL);// 1170170074
        for(i=0;i<(t-1170170074);i++) rand();
}/**/

void dizpet(long i){/* shifts cursor on the screen i characters back */
        long j;
        char k=8;
        for(j=0;j<i;j++) fprintf(stdout,"%c",k);
}


void dataHistoParams (long Samples, double* Data, double* p){
	//Computes three basucoarameters for histogram. No rounding.
	//p[0]=bins
	//p[1]=LeftRegularMargin
	//p[2]=RightRegularMargin
	long i;
	double x;
	p[0]=(long)(sqrt((double)Samples)*0.5);//number of regular bins
	p[1]=1.0e33;
	p[2]=-1.0e33;
	for(i=0;i<Samples;i++){
		x=Data[i];
		if(x<p[1]) p[1]=x;
		if(p[2]<x) p[2]=x;
	}	
}

void EvalHisto (long Samples, double* Data, double* Margins, double* Bin, double* p){
/*	Generates histogram into Margins and Bin arrays from Data with parameters p.	
	Data is supposed to lie between -1.0e33 and +1.0e33.
	Samples        the number of samples to process, i.e. the length of array Data
	Data           one-dimensional real array of a variable 
p[0]	Bins           the number of regular bins (without underflow and overflow bins)
p[1]	LeftRegularMargin     left margin of regular bins
p[2]	RightRegularMargin    right margin of highest regular bin
    Output vectors of results:
	Margins        one-dimensional double array of left bin margins
	Bin            one-dimensional long array of individual bin counts
    Bin organization: underflow bin, regular bins (bin count of them), overflow bin.\n");
    if no LeftMargin or RightMargin is given then corresponding under- or over- flow bins are empty.*/
	long i,j,Bins;
	double xx,binsize;
	Bins=(long)(p[0]+0.001);
	//underflow bin
	Bin[0]=0;
	Margins[0]=-1.0e33;
	binsize=(p[2]-p[1])/(double)Bins;
	for(j=1;j<=Bins;j++){ //regular bins: 
		Bin[j]=0;
		Margins[j]=p[1]+((double)j-1)*binsize;//(p[2]-p[1])/(double)Bins;
	}
	//correction of the leftmost regular margin
	Margins[1]-=0.0001*binsize;
	//overflow bin
	Bin[Bins+1]=0;
	Margins[Bins+1]=1.0e33;
	//Fill-in histogram:
	for(i=0;i<Samples;i++){
		xx=Data[i];
		for(j=0;j<Bins+2;j++){
			if(Margins[j]<=xx && xx<Margins[j+1]) {
				Bin[j]++;
				break;
			}
		}
	}
}


void autHistoParams(long Samples,  double* p){
	//Computes histogram parameters according to first estimates in p.
	//p[0]=bins
	//p[1]=LeftRegularMargin
	//p[2]=RightRegularMargin
	long i,j,Bins;
	double mint, maxt, mintt, bsize, bbsize;
	mint=(long)(p[1]+0.001);
	maxt=(long)(p[2]+0.001);
	Bins=(long)(p[0]+0.001);//number of regular bins
	//stating left margin
	for(i=10;i>-10;i--){
		for(j=15;j>1;j--){
			mintt=(double)j*Na(10,i);
			if(mintt<=mint) goto a1;
		}
	}
a1:;	//stating binsize
	bsize=(maxt-mintt)/Bins;
	for(i=-10;i<10;i++){
		for(j=1;j<15;j++){//6
			bbsize=(double)j*Na(10,i);
			if(bsize<=bbsize) goto a2;
		}
	}
a2:;	
	bsize=bbsize;
	Bins=(long)(maxt-mintt)/bsize+2;//"2" set empirically
	p[0]=(double)(Bins)+0.001;
	p[1]=mintt;
	p[2]=mintt+Bins*bsize;
}

void WriHisto(FILE* hh, long Samples, double* Data, double* p){
	//if p[0]>0 use p as is
	//if p[0]=0 set p rounded automatically
	//if p[0]<0 set p automatically without rounding
	long i,Bins;
	char tab;
	double cnt;
	tab=9;
	//block prepare for histogram
	//p[0]=bins
	//p[1]=LeftRegularMargin
	//p[2]=RightRegularMargin
	cnt=p[0];	
//printf("IN:   Bins=%lg,  LeftMarg=%lg, RightMarg=%lg\n", p[0],p[1],p[2]);
	if(cnt<1){
		dataHistoParams (Samples, Data, p);
//printf("RAW:  Bins=%lg,  LeftMarg=%lg, RightMarg=%lg\n", p[0],p[1],p[2]);
			if(cnt==0) autHistoParams(Samples, p);
//printf("ROUND:Bins=%lg,  LeftMarg=%lg, RightMarg=%lg\n", p[0],p[1],p[2]);
	}
	Bins=(long)(p[0]+0.001);
	double* Margins = new double[Bins+2];
	double* Bin = new double [Bins+2];
	//END of block prepare for histogram
	EvalHisto (Samples,Data,Margins,Bin,p);
	//write hist
	for(i=0;i<Bins+2;i++){
		fprintf(hh, "%lg%c%lg\n", Margins[i], tab, Bin[i]);//write a line
		fprintf(hh, "%lg%c%lg\n", Margins[i+1], tab, Bin[i]);//write a line
	}
}

void WriDoubleHisto(FILE* hh, long Samples, double* Data, long Samples2, double* Data2, double* p, long norm100){
	//if p[0]>0 use p as is
	//if p[0]=0 set p rounded automatically
	//if p[0]<0 set p automatically without rounding
	long i,j,Bins;
	char tab;
	double cnt,xx;
	tab=9;
	//block prepare for histogram
	//p[0]=bins
	//p[1]=LeftRegularMargin
	//p[2]=RightRegularMargin
	cnt=p[0];	
	if(cnt<1){
		dataHistoParams (Samples, Data, p);
			if(cnt==0) autHistoParams(Samples, p);
	}
	Bins=(long)(p[0]+0.001);
	double* Margins = new double[Bins+2];
	double* Bin = new double [Bins+2];
	//END of block prepare for histogram
	EvalHisto (Samples,Data,Margins,Bin,p);
	//Fill-in second histogram:
	double* Bin2 = new double [Bins+2];
	for(i=0;i<Samples2;i++){
		xx=Data2[i];
		for(j=0;j<Bins+1;j++){
			if(Margins[j]<=xx && xx<Margins[j+1]) Bin2[j]++;
		}
	}
	//normalize bincount to 100:
	if(norm100>0){
		for(i=0;i<Bins+2;i++){
			Bin[i] =100.0*Bin[i]/Samples;
			Bin2[i]=100.0*Bin2[i]/Samples2;
		}
	}
	//write hist
	for(i=0;i<Bins+1;i++){
		fprintf(hh, "%lg%c%lg%c%lg\n", Margins[i],   tab, Bin[i],tab,Bin2[i]);//write a line
		fprintf(hh, "%lg%c%lg%c%lg\n", Margins[i+1], tab, Bin[i],tab,Bin2[i]);//write a line
	}
}

void normalize(double *LearnArr, long Dimension, long LearnSamples, long *Count,
 long classes, double *MeansDev, double Metrics){
    long i, j;
    double trida,x;
	if((long)(Metrics-0.001)!=-2){//not CDM
        for(j=0;j<=Dimension;j++) {MeansDev[0*Dimension+j]=0; MeansDev[1*Dimension+j]=0;}
        //compute means
        for(i=0;i<classes;i++) Count[i]=0;
        for(i=0;i<LearnSamples;i++){
//                printf("%6i",i+1);dizpet(6);
                for(j=0;j<Dimension;j++) MeansDev[0*Dimension+j]+=LearnArr[i*(Dimension+1)+j];
                trida=0.001+LearnArr[i*(Dimension+1)+Dimension];
                Count[(long)trida]++;
        }
        for(j=0;j<Dimension;j++)
                MeansDev[0*Dimension+j]=MeansDev[0*Dimension+j]/LearnSamples;

//		for(j=0;j<Dimension;j++) printf("%lg\n", MeansDev[0*Dimension+j]);
//		getchar();
        //Subtract means and compute std.deviations
        for(i=0;i<LearnSamples;i++) for(j=0;j<Dimension;j++) MeansDev[1*Dimension+j]
			+=Na2(LearnArr[i*(Dimension+1)+j]-MeansDev[0*Dimension+j]);
        for(j=0;j<Dimension;j++) {
			x=sqrt(MeansDev[1*Dimension+j]/(LearnSamples-1));
            if(x<1.0e-30) MeansDev[1*Dimension+j]=1;// the case of constants in the learning set.
            else MeansDev[1*Dimension+j]=x;
        }

//		for(j=0;j<Dimension;j++) printf("%lg\n", MeansDev[0*Dimension+j]);
//		getchar();

		if((long)(Metrics-0.001)==-1){//Mahalanobis independent of class
			for(j=0;j<Dimension;j++) MeansDev[1*Dimension+j]=1;//std.dev not modified
		}

        //Standardize the LearnArr; the array LearnArr will be overriden by normalized variables:
        for(i=0;i<LearnSamples;i++){
			for(j=0;j<Dimension;j++) LearnArr[i*(Dimension+1)+j]=(LearnArr[i*(Dimension+1)+j]
				-MeansDev[0*Dimension+j])/MeansDev[1*Dimension+j];
//			printf("%lg  %lg  %lg\n",LearnArr[i*(Dimension+1)+0],LearnArr[i*(Dimension+1)+1],LearnArr[i*(Dimension+1)+2]);

		}
//		getchar();
	}
	else{// CDM <=> Metrics==-2
        for(j=0;j<=Dimension;j++) {MeansDev[0*Dimension+j]=0; MeansDev[1*Dimension+j]=0;}
        //compute means
        for(i=0;i<classes;i++) Count[i]=0;
        for(i=0;i<LearnSamples;i++){
//                printf("%6i",i+1);dizpet(6);
                for(j=0;j<Dimension;j++) MeansDev[0*Dimension+j]+=LearnArr[i*(Dimension+1)+j];
                trida=0.001+LearnArr[i*(Dimension+1)+Dimension];
                Count[(long)trida]++;
        }
        for(j=0;j<Dimension;j++)
                MeansDev[0*Dimension+j]=MeansDev[0*Dimension+j]/LearnSamples;

//		for(j=0;j<Dimension;j++) printf("%lg\n", MeansDev[0*Dimension+j]);
//		getchar();

        //Subtract means and compute std.deviations
        for(i=0;i<LearnSamples;i++) for(j=0;j<Dimension;j++) MeansDev[1*Dimension+j]
			+=Na2(LearnArr[i*(Dimension+1)+j]-MeansDev[0*Dimension+j]);
        for(j=0;j<Dimension;j++) {
			x=sqrt(MeansDev[1*Dimension+j]/(LearnSamples-1));
            if(x<1.0e-30) MeansDev[1*Dimension+j]=1;// the case of constants in the learning set.
            else MeansDev[1*Dimension+j]=x;
        }

//		for(j=0;j<Dimension;j++) printf("%lg\n", MeansDev[0*Dimension+j]);
//		getchar();

		//if((long)(Metrics-0.001)==-2){//Mahalanobis dependent of class
			for(j=0;j<Dimension;j++) MeansDev[1*Dimension+j]=1;//std.dev not modified
		//}

        //Standardize the LearnArr; the array LearnArr will be overriden by normalized variables:
        for(i=0;i<LearnSamples;i++){
			for(j=0;j<Dimension;j++) LearnArr[i*(Dimension+1)+j]=(LearnArr[i*(Dimension+1)+j]
				-MeansDev[0*Dimension+j])/MeansDev[1*Dimension+j];
//			printf("%lg  %lg  %lg\n",LearnArr[i*(Dimension+1)+0],LearnArr[i*(Dimension+1)+1],LearnArr[i*(Dimension+1)+2]);

		}
//		getchar();
	}
}




long EvalROC (long Samples, double *Responses, double *TrueVal, double threshold, long *OrderNo, double *SigEffi, double *BackErr, double *ClasError, double *RejFact, double *Enrichm, double *QualFact, double *MinClasErr) {
	/*
	Returns Number-of-errors for min.clas.error when OK else 0 - it means less than 2 samples or samples of one class are missing.

	Samples        number of samples to process, i.e. the length of arrays Responses, TrueVal, and all others.
	Responses      one-dimensional double array of responses of the classifier; rows correspond to individual samples. 
	TrueVal        one-dimensional double array of true class values for individual samples in the same order as in the Responses  
	threshold      if for a sample the TrueVal is larger or equal to the threshold, the sample belongs to the signal class
				   else it belongs to the background class. 
	OrderNo        one-dimensional integer array of original order numbers of samples after processing
	SigEffi        one-dimensional double array of values of signal efficiency (acceptance) 
	BackErr        one-dimensional double array of ... etc.  
	ClasError      one-dimensional double array of ... [use items 1...(ROCcount-2) only, not 0...(ROCcount-1) !]
	RejFact        one-dimensional double array of ... [use items 1...(ROCcount-2) only, not 0...(ROCcount-1) !] 
	Enrichm        one-dimensional double array of ... [use items 1...(ROCcount-2) only, not 0...(ROCcount-1) !]  
	QualFact       one-dimensional double array of ... [use items 1...(ROCcount-2) only, not 0...(ROCcount-1) !]
	MinClasErr     one-dimensional double array with three cells only ... minimal classification error [0], corresponding response or threshold [1], and area above the ROC curve [2].
	*/
	long p, k, CitacVzoru, pom, Sum1, Sum1tot, Sum0, ROCcount;
	double ddd, SigEff, BckErr;
	
	// Count signals in a file and enumerate them:
	Sum1tot=0;
    for(CitacVzoru=0; CitacVzoru<Samples; CitacVzoru++) {
		if(TrueVal[CitacVzoru]>=threshold) Sum1tot++;
		OrderNo[CitacVzoru]=CitacVzoru+1;
	}
	//Check of valid data:
	if(!(Samples>1 && Sum1tot>0 && Sum1tot<Samples)) return 0;

	//Sort according to responses in ascending order by bubblesort:
	p=1; k=Samples;
	while(p==1){  
		p=0; k-=1;
		for(CitacVzoru=0;CitacVzoru<k;CitacVzoru++){//internal cycle over samples
			if (Responses[CitacVzoru+1] < Responses[CitacVzoru]){//exchange

				ddd=Responses[CitacVzoru];
				Responses[CitacVzoru]=Responses[CitacVzoru+1];
				Responses[CitacVzoru+1]=ddd;
				
				ddd=TrueVal[CitacVzoru];
				TrueVal[CitacVzoru]=TrueVal[CitacVzoru+1];
				TrueVal[CitacVzoru+1]=ddd;
				
				pom=OrderNo[CitacVzoru];
				OrderNo[CitacVzoru]=OrderNo[CitacVzoru+1];
				OrderNo[CitacVzoru+1]=pom;
				p=1;
			}
		}
	}

	//compute arrays SigEffi and BckErr and others:
	Sum1=0; Sum0=0; SigEff=0; BckErr=0;	MinClasErr[0]=1.0e33;
    for(CitacVzoru=0; CitacVzoru<Samples; CitacVzoru++){//loop over samples in already sorted arrays
		//start of computation
		if(TrueVal[CitacVzoru]>=threshold) Sum1++;else Sum0++;
		SigEff=(double)(1.0-(double)Sum1/(double)Sum1tot);
		BckErr=(double)(1.0-(double)(CitacVzoru-Sum1)/(Samples-Sum1tot));
		BackErr[CitacVzoru]=BckErr;
		SigEffi[CitacVzoru]=SigEff;
		ClasError[CitacVzoru]=(1-SigEff+BckErr)/2;
		//ClasError[CitacVzoru]=((double)(Sum1+ (Samples-Sum1tot)-Sum0))/((double)Samples);
		//ClasErr = (NoOfSig + NoOfBckTot - NoOfBck) / (NoOfSigTot + NoOfBckTot)
		if(MinClasErr[0]>ClasError[CitacVzoru]) {
			MinClasErr[0] = ClasError[CitacVzoru];
			MinClasErr[1] = Responses[CitacVzoru];
		}
		RejFact[CitacVzoru]=BckErr!=0?1.0/BckErr:0.0;
		Enrichm[CitacVzoru]=BckErr!=0?SigEff/BckErr:0.0;
		QualFact[CitacVzoru]=BckErr!=0?SigEff*sqrt(1/BckErr):0.0;
	}

	//recompute all arrays
	ROCcount=0;

	BackErr[ROCcount]=1;
	SigEffi[ROCcount]=1;
	RejFact[ROCcount]=1;
	Enrichm[ROCcount]=1;
	QualFact[ROCcount]=1;
	ClasError[ROCcount]=1;
	ROCcount++;
/**/
	for(CitacVzoru=1; CitacVzoru<Samples-1; CitacVzoru++){//loop over samples in already sorted arrays
		if(BackErr[CitacVzoru]<BackErr[CitacVzoru-1] && SigEffi[CitacVzoru]>SigEffi[CitacVzoru+1]) {//write next values 
			BackErr[ROCcount]=BackErr[CitacVzoru];
			SigEffi[ROCcount]=SigEffi[CitacVzoru];
			RejFact[ROCcount]=RejFact[CitacVzoru];
			Enrichm[ROCcount]=Enrichm[CitacVzoru];
			QualFact[ROCcount]=QualFact[CitacVzoru];
			ClasError[ROCcount]=ClasError[CitacVzoru];
			ROCcount++;
		}
	}/**/

	BackErr[ROCcount]=0;
	SigEffi[ROCcount]=0;
	RejFact[ROCcount]=0;
	Enrichm[ROCcount]=0;
	QualFact[ROCcount]=0;
	ClasError[ROCcount]=1;
	ROCcount++;
/**/
	//area below the ROC curve
	MinClasErr[2]=0;
	for(k=1; k<ROCcount; k++) MinClasErr[2]+=0.5*(BackErr[k-1]-BackErr[k])*(2-SigEffi[k-1]-SigEffi[k]);

	return (long)(Samples*MinClasErr[0]+0.0001);
}






/* cte radek dim reals do pole a[] a vraci 0 kdyz ok, -1 pri chybe*/
long CtiRadekAlabel(FILE *gg, double *a, long dim, char *label) {
        long i;
        for(i=0;i<dim;i++){
                a[i]=0;
                if(!fscanf(gg,"%lg", &a[i])) return -1;
        }
        MYstrcpy(label, NULL , 0, 0, 0);
        if(!fscanf(gg,"%s", label)) return -1;
        return 0;
}

/* cte radek dim reals do pole a[] a vraci 0 kdyz ok, -1 pri chybe
   Reading pairs (index of word, word count) until (-1 -1) then Class string.*/
long CtiRadekAlabel2(FILE *gg, double *a, long dim, char *label) {
        long i, index, indexold; double x;
        indexold=-1;//////////////
        for(i=0;i<dim;i++) a[i]=0;
        for(;;){
                if(!fscanf(gg,"%li%lg", &index, &x)) return -1;
                if(index<0) break;
                if(index==indexold) return 0;
                indexold=index;
                if(index<dim) a[index]=x;
        }
        MYstrcpy(label, NULL, 0, 0, 0);
        if(!fscanf(gg,"%s", label)) return -1;
        MYstrncpy(label, label, 2);//MYstrncpy(char * s1, char * s2, size_t n);
        return 0;
}

long CtiRadek(FILE *gg, double *a) { /* cte radek reals do pole a[] a vraci pocet prectenych cisel, 0 kdyz zadne, -1 pri chybe */
        long CitUdaju, bylaMez, ii, strrCit, xx;
        char st[50000],strr[50], ch[1];
        if(!fscanf(gg,"%[^\n]\n", st)) return -1; /*peten cel dek jako etzec*/
        strcat(st,"   ");//pidme koncov mezery
        CitUdaju=0;
        strrCit=0;
        bylaMez=0;
        strcpy(ch, "");
        strcpy(strr,"");//vyprazdnime pro novy udaj
        xx=(long)strlen(st);
        for (ii=0;ii<xx;ii++) {//cykl po jednotlivch znacch
                if(st[ii]==32 || st[ii]==9 || st[ii]==59) {
                        if(bylaMez || strrCit==0) ;//kdyz po mezere prijde mezera nebo tabelator, tak nic
                        else {//nebyla mezera a prisla mezera, tedy konec skupiny v strr a to je novy udaj.
                                strr[strrCit]=ch[0];
                                sscanf(strr,"%lg",&(a[CitUdaju]));//sprintf(strr,"%lg",a[CitUdaju]);
                                CitUdaju++;
                                bylaMez=1;
                                strcpy(strr,"");//vyprazdnime pro novy udaj
                                strrCit=0;
                        }
                }
                else {//proste prisel novy znak, pridame ho k predeslym
                        if(st[ii]==44) st[ii]=46;// desetinnou carku nahradime teckou)*+,-./01789:;568<;
                        MYstrncpy(&strr[strrCit],&st[ii],1);
                        strrCit++;
                        bylaMez=0;
                        //strncpy(ch,&st[ii],1);
                        //strcat(strr,ch);

                }  // na konci je v CitUdaju p[ocet a v a[] jednotlive udaje

        }
        return CitUdaju;
}

//search for label index in labList
long LabelIndex(char *labList, char *label, long labels){
        long i; char lb[20];//LabLen
        for(i=0;i<labels;i++){
                MYstrcpy(lb, labList, 0, 20*i, 20-1);
                if(MyStrCmp(label, lb)) break;
        }
        return i;
}

//search for label in labList according to its index
void LabelByIndex(char *labList, char *label, long labels, long index){
        MYstrcpy(label, labList, 0, 20*index, 20-1);
}


void CountFile(char *SouborSite, long FileType, long Vstupu, long &labels, char *labList, long &RadkuSite, double *a, long MaxClasses, long TAResp) {
        long i, j, lab, LabLen;
        char nic[10], label[20], lb[20];//LabLen
        FILE *ff;

        LabLen=20;
        lab=labels;
        if ((ff = fopen(SouborSite, "rt")) == NULL){
                fprintf(stderr, "Procedure CountFile: Cannot open file to read %s.\n", SouborSite);
                exit(1);
        }

        if(FileType==1 && lab==0) {
                for(i=0;i<MaxClasses*LabLen;i++) labList[i]=(char)NULL;
        }
        //labels=0;
        RadkuSite=0;
        for(i=0;;i++){  //for(i=0;i<100000;i++){
//                printf("%6li ",i+1);//dizpet(6);

                if(FileType==0) {
                        j=CtiRadek(ff, a);// here I need number of labels "labels" only
                        sprintf(label,"%lg", a[Vstupu]);
                }

                if(FileType==1){
						if(TAResp==1) if(0>CtiRadekAlabel(ff, a, Vstupu, label)) return;
						if(TAResp==0) j=CtiRadek(ff, a);
                }

                if(FileType==2){
                        if(0>CtiRadekAlabel2(ff, a, Vstupu, label)) break;
                }

                if(MyStrCmp(label,strcpy(nic,""))) goto EndLrnFile1;
                RadkuSite++;

                if(lab==0){
                        if(labels==0){strcpy(labList, label); labels++;}
                        else {
                                for(j=0;j<labels;j++){
                                        MYstrcpy(lb, labList, 0, 20*j, 19);
                                        if(MyStrCmp(label, lb)) {
                                                break;
                                        }
                                }
                                if(j==labels) {//new label
                                        MYstrcpy(labList, label, 20*labels, 0, strlen(label));
                                        if(labels>=MaxClasses) {
                                                printf("Procedure CountFile: Exceeded max number of labels possible (%li).\n", MaxClasses);
                                                return;
                                        }
                                        labels++;
                                }
                        }
                }

                if(feof(ff)) break;
        }
EndLrnFile1:;
        fclose(ff);
        //RadkuSite=i+1;
}





void ReadFile(char *SouborSite, long FileType, long Vstupu, long &labels, char *labList, long &RadkuSite, double *a, double *NN, long *Count, long TAResp){
//      ReadFile(SouborSite, FileType, Vstupu, labels, LabList, RadkuSite, a, NN);
        long i, j, tst, LabLen;
        char nic[10], label[20];//LabLen
        FILE *ff;
        tst=Count[0];//when reading testing file, we already have nonzero samples count according to labels.
        LabLen=20;
        if ((ff = fopen(SouborSite, "rt")) == NULL){
                fprintf(stderr, "Cannot open file of Network.\n");
                exit(1);
        }

        for(i=0;i<RadkuSite;i++){
//                printf("%6li ",i+1);//dizpet(6);

                if(FileType==0) {
                        j=CtiRadek(ff, a);// here I need number of labels "labels" only
                        if(j<Vstupu+TAResp)
                                goto EndLrnFile2;
                        if(TAResp==1) sprintf(label,"%lg", a[Vstupu]);
			else sprintf(label,"%li", labels);
                }

                if(FileType==1){
			if(TAResp==1) if(0>CtiRadekAlabel(ff, a, Vstupu, label)) return;
			if(TAResp==0) j=CtiRadek(ff, a);
                }

                if(FileType==2){
                        if(0>CtiRadekAlabel2(ff, a, Vstupu, label)) return;
                }

                if(tst==0){//learning file. We must count samples according to labels.
                        if(MyStrCmp(label, strcpy(nic,""))) goto EndLrnFile2;//NULL
                        for(j=0;j<=Vstupu;j++){            //we must save all inputs and the output!
                                NN[i*(Vstupu+1)+j]=a[j];
                        }
                        //long LabelIndex(char labList[], char label[20], long labels){
                        j=LabelIndex(labList, label, labels);
                        NN[i*(Vstupu+1)+Vstupu]=j;
                        Count[j]++;
                }
                else {//testing file need not have labels
                        if(TAResp==0) a[Vstupu]=0; //when there are no labels in the testfile  MyStrCmp(label, "")
                        else a[Vstupu]=LabelIndex(labList, label, labels);
                        for(j=0;j<=Vstupu;j++) NN[i*(Vstupu+1)+j]=a[j];
                }

                if(feof(ff)) break;
        }
EndLrnFile2:;
        fclose(ff);
}


int FileRowsCount(FILE *ggg) {
	//if OK returns No.of rows of given file from the point where it stays
	//else returns -1.
	int i;
	char *st;
	st=new char[5000];
	
	for(i=1;;i++){  //for(i=1;i<100000000;i++){
		if(!fscanf(ggg,"%[^\n]\n", st)) return -1; //read the whole row as a string
		if(feof(ggg)) break;
	}
	delete st;
	return i;
}


int FillArray(FILE *gg, double *Arr, int rows, int cols, int emptyRows){
	//reads rows organized file to array
	//at the beginning it jumps over emptyRows lines
	//if ok returns true No. of rows, else -1
	int i, j, n;
	char *st;
	double *pA;

	st=new char[500];
	pA = new double[cols+10];
	for(i=0;i<emptyRows;i++){
		if(!fscanf(gg,"%[^\n]\n", st)){//read the whole row as a string
			i=-1; 
			goto x1;
		}
		if(feof(gg)){
			i=-1;
			goto x1;
		}
	}
	for(i=0;i<rows;i++){
		n=CtiRadek(gg, pA);
		if(n==-1){
			printf("Error filling array in FillArray procedure\n");
			i=-1;
			goto x1;
		}
		for(j=0;j<cols;j++){
			if(j<n) Arr[i*cols+j]=pA[j];
			else Arr[i*cols+j]=0;//fill-in zeros if necessary
		}
		if(feof(gg)) break;
	}
x1:
	delete pA;
	delete st;
	return i;
}



double Sigmoida(double x, double MinZ, double MaxZ){
        double y,x1,aa, sigm;
        aa=fabs(MinZ)+fabs(MaxZ);
        if ((aa > 1.0E+20)||(fabs(aa-1111)<1.0e-10))
        sigm = x;
        else {
        if (fabs(aa-2222)<1.0e-10)
                sigm = sqrt(fabs(x))*sign(x);
        else {
                if (fabs(aa-3333)<1.0e-10) {
                if (fabs(x-0)<1.0e-10)
                        sigm = 0;
                else
                        sigm = exp(log(fabs(x))/3)*sign(x);
                }
                else {
                        x1 = x*2/(MaxZ - MinZ);/*prepocet na long. <-1 +1>.*/
                        if (x1 >  80) x1 =  80;
                        if (x1 < -80) x1 = -80;
                        y = exp(-x1);
                        y = (1-y)/(1+y);
                        sigm = y * (MaxZ - MinZ)*0.5 + 0.5*(MinZ+MaxZ);
                }
        }
        }
        return sigm;
}

double InvSigmo(double x, double MinZ, double MaxZ) {
  double y, aa;
  aa=fabs(MinZ)+fabs(MaxZ);
  if ((aa > 1.0E+20)||(fabs(aa-1111)<1.0e-10)) return x;
  else{
    if (fabs(aa-2222)<1.0e-10) return x*x*sign(x);
    else {
      if (fabs(aa-3333)<1.0e-10) return x*x*x;
      else {
        y = 2 * (x - 0.5*(MinZ+MaxZ)) / (MaxZ - MinZ);
        if (y > 0.999999) y = 0.999999;
        if (y <-0.999999) y =-0.999999;  /*sigmoida je zde oboustranna, tj -1 az +1*/
        y = 0.5*log( (1+y)/(1-y) );      /*interval <-1 +1>.*/
        return y*0.5*(MaxZ - MinZ);
      }
    }
  }
}

void EvalWri(char *program, char *SouborSite, char *TestFile, long FileType, long labels, 
			 long Vstupu, double *TST, long RadkuTST, long *CountErr, double *Probab, char *labList,
			 char *text, double DME, long SEL, double Metrics){
		FILE *gg;
		char tab, c, SouborRes[255], res[4], *ptr, labela[20], labelXresult[20]; 
		long i, j;
		double prob, found, soll;
		strcpy(res,".RES");
		//generate response file *.res
		tab=9;    
		c='.';
		ptr = strrchr(TestFile, c);
		if (ptr-TestFile>0){
			if (ptr) MYstrncpy(SouborRes,TestFile,ptr-TestFile);
			else strcpy(SouborRes,TestFile);
		}
		//else strcpy(SouborRes,"TestFile");
		else strcpy(SouborRes,TestFile); //20150804
		strcat(SouborRes,res);
		//Open response file *.res
		if ((gg = fopen(SouborRes, "wt")) == NULL){
				fprintf(stderr, "Cannot open file of Results.\n");
				exit(1);
		}
		if(Metrics!=0)//change 22.3.2017 
			fprintf(gg, "%s : %s SEL=%li, lrn file=%s, tst file=%s, inputs=%li, FileType=%li, Metrics=%lg\n",
				program, text, SEL, SouborSite,TestFile,Vstupu,FileType, Metrics);
		else //for e.g. Bayes approach not using any metrics in data space
			fprintf(gg, "%s : %s SEL=%li, lrn file=%s, tst file=%s, inputs=%li, FileType=%li\n",
				program, text, SEL, SouborSite,TestFile,Vstupu,FileType);

//	printf("\nMyFu1385: Evaluate and write results %li\n",RadkuTST);
		CountErr[0]=0;
		for(i=0;i<RadkuTST;i++){
				soll=TST[i*(Vstupu+1)+Vstupu];

				prob=0.0; found=0;
				for(j=0;j<labels;j++) {
						if(Probab[i*labels+j]>prob) {
								prob=Probab[i*labels+j];//hodnota maxima
								found=(double)j;  //misto (index=trida) maxima
						}
				}

				if(fabs(found-soll)>0.1) CountErr[0]++;

				for(j=0;j<Vstupu;j++) fprintf(gg, "%lg%c",TST[i*(Vstupu+1)+j], tab);
				LabelByIndex(labList, labela, labels, (long)(0.001+soll));
				LabelByIndex(labList, labelXresult, labels, (long)(0.001+found));
				fprintf(gg, "%s%c%s%c", labela, tab, labelXresult, tab);
				for(j=0;j<labels;j++) fprintf(gg, "%lg%c",Probab[i*labels+j], tab);
				fprintf(gg, "%li\n", i+1);
				printf(     "%s%c%s%c%li\n", labela, tab, labelXresult, tab, i+1);
		}
		fclose(gg);
}



void registLong(char *program, char *SouborSite, char *TestFile, long RadkuTST, long labels, long *CountErr, 
			double Metrics, char *text, double DME, long SEL){
	//registers into register.txt file by append: 
	//either errors found (the first cell of CountErr is nonzero, the other zero)
	//or for cases with output meaning the probability the No. of cases above the following 16 probability levels:
	//0.40 0.42 0.44 0.46 0.48 0.50 0.52 0.54 0.56 0.58 0.60 0.62 0.64 0.66 0.68 0.7
	//The Nos. of cases are stored in CountErr array
	FILE *hh;
	char tab;
	long i, sum;
	if ((hh = fopen("register.txt", "at")) == NULL){
            fprintf(stderr, "Cannot open file ""register.txt"".\n");
            exit(1);
    }
    tab=9;
	sum=0; for(i=0;i<16;i++) sum+=CountErr[i]!=0?1:0;
	fprintf(hh, "%s%c%s%cSEL=%c%li%c%s%c%s%c%li%c%5.2lf%c%li", 
		program, tab, text, tab, tab, SEL, tab, SouborSite, tab, TestFile, tab, 
		labels, tab, Metrics, tab, RadkuTST);
	printf     ("%s%c%s%cSEL=%li%c%s%c%s%cclasses=%li%cMetric=%5.2lf%ctest samples=%li", 
		program, tab, text, tab, SEL, tab, SouborSite, tab, TestFile, tab, 
		labels, tab, Metrics, tab, RadkuTST);
	if(sum<1) sum=1;//to print even if there is zero error
	for (i=0;i<sum;i++){
		fprintf(hh, "%c%li", tab, CountErr[i]);
		printf (    "%cerrors=%li", tab, CountErr[i]);
	}
		fprintf(hh, "\n");
		printf (    "\n");
	fclose(hh);
}

void regist(char *program, char *SouborSite, char *TestFile, long RadkuTST, long labels, long *CountErr, 
			double Metrics, char *text){
	//registers into register.txt file by append: 
	//either errors found (the first cell of CountErr is nonzero, the other zero)
	//or for cases with output meaning the probability the No. of cases above the following 16 probability levels:
	//0.40 0.42 0.44 0.46 0.48 0.50 0.52 0.54 0.56 0.58 0.60 0.62 0.64 0.66 0.68 0.7
	//The Nos. of cases are stored in CountErr array
	FILE *hh;
	char tab;
	long i, sum;
	if ((hh = fopen("register.txt", "at")) == NULL){
            fprintf(stderr, "Cannot open file ""register.txt"".\n");
            exit(1);
    }
    tab=9;
	sum=0; for(i=0;i<16;i++) sum+=CountErr[i]!=0?1:0;
	fprintf(hh, "%s%c%s%c%s%c%s%c%li%c%5.2lf%c%li", 
		program, tab, text, tab, SouborSite, tab, TestFile, tab, 
		labels, tab, Metrics, tab, RadkuTST);
	printf     ("%s%c%s%c%s%c%s%cclasses=%li%cMetric=%5.2lf%ctest samples=%li", 
		program, tab, text, tab, SouborSite, tab, TestFile, tab, 
		labels, tab, Metrics, tab, RadkuTST);
	if(sum<1) sum=1;//to print even if there is zero error
	for (i=0;i<sum;i++){
		fprintf(hh, "%c%li", tab, CountErr[i]);
		printf (    "%cerrors=%li", tab, CountErr[i]);
	}
		fprintf(hh, "\n");
		printf (    "\n");
	fclose(hh);
}


void howfar(long RadkuTST, long radek){
	//Checks how the program run has proceed
	//writes into howfar.txt file by append: 
	//percentage of work, program name, total testing rows, rows already computed, 
	FILE *hh;
	char tab;
	double percent;
	if ((hh = fopen("howfar.txt", "wt")) == NULL){
            fprintf(stderr, "Cannot open file ""howfar.txt"".\n");
            exit(1);
    }
    tab=9;
	percent=100*((double)radek+1)/(double)RadkuTST;
	fprintf(hh, "%7.3lf %%, i.e. %c%li of %c%li", 
		percent, tab, radek+1, tab, RadkuTST);
		fprintf(hh, "\n");
	fclose(hh);
}

void howfarWhat(long RadkuTST, long radek, char *Text){
	//Checks how the program run has proceed
	//writes into howfar.txt file by append: 
	//percentage of work, program name, total testing rows, rows already computed, 
	FILE *hh;
	char tab;
	double percent;
	if ((hh = fopen("howfar.txt", "wt")) == NULL){
            fprintf(stderr, "Cannot open file ""howfar.txt"".\n");
            exit(1);
    }
    tab=9;
	percent=100*((double)radek+1)/(double)RadkuTST;
	fprintf(hh, "%s:%c%7.3lf %%, i.e. %c%li of %c%li\n", 
		Text,tab,percent, tab, radek+1, tab, RadkuTST);
//		fprintf(hh, "\n");
	fclose(hh);
}

void primsg(char *Text){
	//Writes Text into file message.txt
	FILE *hh;
	if ((hh = fopen("message.txt", "wt")) == NULL){
            fprintf(stderr, "Cannot open file ""message.txt"".\n");
            exit(1);
	}
	fprintf(hh, "%s\n", Text);
	fclose(hh);
}


void CovarMatrix(double *LearnArr, long Dimension, long LearnSamples, long Class, double *CovMat, long *Count) {
	//Computes the covariance matrix CovMat for Class or for all samples if Class<0.
	//zero means are supposed !!  
	long i,j,k,kk,trida;
	double x;
	if(Class<0) {
		x=1/(double)LearnSamples;
		kk=0;
	}
	else {
		x=1/(double)Count[Class];
		kk=Class;
	}
	//zeros to kk-th layer of CovMat
	for(i=0;i<Dimension;i++)
		for(j=0;j<Dimension;j++)
			CovMat[kk*Dimension*Dimension+i*Dimension+j]=0;
	
	//fill-in the CovMat (the class corresponding layer)
	for(k=0;k<LearnSamples;k++){
		trida=(long)LearnArr[k*(Dimension+1)+Dimension];
		if(trida==Class || Class<0) {
			for(i=0;i<Dimension;i++)
				for(j=0;j<Dimension;j++){
					CovMat[kk*Dimension*Dimension+i*Dimension+j]
					    +=LearnArr[k*(Dimension+1)+i]*LearnArr[k*(Dimension+1)+j]*x;
				}
		}
	}
}// end of void CovarMatrix.


double MahalanobisDist(double *x, double *y, double *ZCovMat, long Dimension, long Class){
	//computes Mahalanobis dostance between x and y using CovMat already known
	//via solution of system of linear equations
	//zero means are supposed !!  
	long i, j, kk;
	double xx;
	double* ZZ= new double [Dimension*Dimension];// working ZCovMat, i.e. 
	if(Class<0) kk=0; else kk=Class;
	for(i=0;i<Dimension;i++){
		for(j=0;j<Dimension;j++){
			ZZ[i*Dimension+j]=ZCovMat[kk*Dimension*Dimension+i*Dimension+j];
//			printf("%10lg  ", CM[i*Dimension+j]);
		}
//		printf("\n");
	}
//	getchar();
	//double mahala(double u[], double v[], double z[], long n){
	xx=mahala(x, y, ZZ, Dimension);
	delete ZZ;  // <<<<NUTNE!!!
	return xx;
}

double Orloci(double *x, double *y, long Dimension){// = sqrt(2*(1-<x,y>/(||x||*||y||))
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++){
		xx+=Na2(x[i]);
		yy+=Na2(y[i]);
		xy+=x[i]*y[i];
	}
	xx=sqrt(xx);
	yy=sqrt(yy);
	return sqrt(2*(1-xy/(xx*yy)));
}


double Angular(double *x, double *y, long Dimension){//= arccos(<x,y>/(||x||*||y||))
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++){
		xx+=Na2(x[i]);
		yy+=Na2(y[i]);
		xy+=x[i]*y[i];
	}
	xx=sqrt(xx);
	yy=sqrt(yy);
	return acos(xy/(xx*yy));
}

double Clark(double *x, double *y, long Dimension){//Clark metrics = sqrt{1/nSUM[(xi-yi)/(|xi|+|yi|)]^2}
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) {
		if(fabs(x[i])+fabs(y[i])) xx+=Na2((x[i]-y[i])/(fabs(x[i])+fabs(y[i])));
//printf("Clark %lg %lg %lg\n",xx,x[i], y[i]);
	}
//getchar();
	return sqrt(xx/Dimension);
}

double Lorentz(double *x, double *y, long Dimension){//Lorentz metrics = SUM[ln(1+|xi-yi|)]
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) xx+=log(1+fabs(x[i]-y[i]));
	return xx;
}


double Canberra(double *x, double *y, long Dimension){//Canberra metrics = SUM[|xi-yi|/(|xi|+|yi|)]
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) {
		xy=fabs(x[i])+fabs(y[i]);
		if(xy>1.0e-10) xx+=fabs(x[i]-y[i])/(xy);
	}
	return xx;
}


double BrayCurtis(double *x, double *y, long Dimension){//Bray-Curtis (nearly) metrics = SUM|xi-yi|/SUM(xi+yi) 
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) {
		xx+=fabs(x[i]-y[i]);
		yy+=x[i]+y[i];
	}
	return xx/yy;
}


double Intersection(double *x, double *y, long Dimension){//Intersection (nearly) metrics = 1-SUM[min(xi,yi)]/min[sum xi,sum yi]
	long i;
	double r,xx,yy,xy,xi,yi;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) {
		xi=fabs(x[i]);
		yi=fabs(y[i]);
		r=xi<yi?xi:yi;
		xy+=r;
		xx+=xi;
		yy+=yi;
	}
	r = (xx<yy?xx:yy);
	if(fabs(r)>0.0000000001) return 1-xy/r;
	else return 1.0;
}


double CayleyKleinHilbert(double *x, double *y, long Dimension){
	//Cayley-Klein-Hilbert metrics = arccosh{abs[1-<x,y>]/[sqrt abs(1-<x,x>)*sqrt abs(1-<y,y>)]
	long i;
	double xx,yy,xy,xi,yi,r,coef;
	coef=0.5;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) {
		xi=fabs(x[i]*coef);
		yi=fabs(y[i]*coef);
		xy+=xi*yi;
		xx+=Na2(xi);
		yy+=Na2(yi);
	}
	coef=1.0/((double)(Dimension*Dimension));
	r= arccosh(fabs(1-xy*coef)/(sqrt(fabs(1-xx*coef))*sqrt(fabs(1-yy*coef))));
//printf("Cayley %lg %lg %lg %lg\n",xx*coef,yy*coef,xy*coef,r);
//getchar();
	if(fabs((1-xx*coef)*(1-yy*coef))>0.0000000001) return r;
	else return 1.0;
}


double Weierstrass(double *x, double *y, long Dimension){
	//Weierstrass metrics = arccosh{abs[sqrt(1+<x,x>)*sqrt(1+<y,y>)-<x,y>]}
	long i;
	double xx,yy,xy;
	xx=0; yy=0; xy=0;
	for(i=0;i<Dimension;i++) {
		xy+=x[i]*y[i];
		xx+=Na2(x[i]);
		yy+=Na2(y[i]);
	}
	return arccosh(fabs(sqrt(1+xx)*sqrt(1+yy)-xy));
}


double Hassanat(double *x, double *y, long Dimension){
	// Hassanat metrics = 
        // = sum{if min xi,yi>0 then 1-(1+min xi,yi)/(1+max xi,yi) else 1-(1+min xi,yi + abs min xi,yi)/(1+max xi, yi +abs min xi,i)}
	long i;
	double dist,minx,maxx;
	dist=0;
	for(i=0;i<Dimension;i++) {
		minx=x[i]<y[i]?x[i]:y[i];
		maxx=x[i]>y[i]?x[i]:y[i];
		//dist+=minx>=0?1-(1+minx)/(1+maxx):1-(1+minx+fabs(minx))/(1+maxx+fabs(minx));
		dist+=( minx>=0?1.0-(1.0+minx)/(1.0+maxx):1.0-1.0/(1.0+maxx+fabs(minx)) );
	}
	return dist/Dimension;
}

//added 9.4.2016>
double PearsonCorrel(double *x, double *y, long Dimension, long suppressI){
	//with suppressI coordinate suppressed, i.e. deleted.
	//No deletion, use suppressI=-1
	double xpruh, ypruh,cit, jm1,jm2;
	long i;
	cit=0; jm1=0; jm2=0;xpruh=0;ypruh=0;
	for(i=0;i<Dimension && i!=suppressI;i++){
		xpruh+=x[i];
		ypruh+=y[i];
	}
	xpruh/=Dimension-suppressI;
	ypruh/=Dimension-suppressI;
	for(i=0;i<Dimension && i!=suppressI;i++){
		cit+=(x[i]-xpruh)*(y[i]-ypruh);
		jm1+=Na2(x[i]-xpruh);
		jm2+=Na2(y[i]-ypruh);
	}
	if(jm1*jm2>0.0000000001) return cit/(sqrt(jm1)*sqrt(jm2));
	else return 05.0;
}

double PearsonDist(double *x, double *y, long Dimension){
	return 1-PearsonCorrel(x, y, Dimension, -1);
}


double JacknifeCorrel(double *x, double *y, long Dimension){
	double rho,rh;
	long i;
	rho=0;
	for(i=0;i<Dimension;i++){
		rh=PearsonCorrel(x,y,Dimension,i);
		if(rh<rho)rho=rh;
	}
	return rho;
}

double JacknifeDist(double *x, double *y, long Dimension){
	return 1-JacknifeCorrel(x, y, Dimension);
}

double GoodmanKruskalCorrel(double *x, double *y, long Dimension){
	double splus,sminus;
	long i;
	splus=0; sminus=0;
	for(i=0;i<Dimension-1;i++){
		if((x[i]<x[i+1] && y[i]<y[i+1]) || (x[i]>x[i+1] && y[i]>y[i+1])) splus++;
		if((x[i]<x[i+1] && y[i]>y[i+1]) || (x[i]>x[i+1] && y[i]<y[i+1])) sminus++;
	}
	return (splus-sminus)/(splus+sminus);
}

double GoodmanKruskalDist(double *x, double *y, long Dimension){
	return 1-GoodmanKruskalCorrel(x, y, Dimension);
}

double KendallCorrel(double *x, double *y, long Dimension){
	double splus,sminus;
	long i;
	splus=0; sminus=0;
	for(i=0;i<Dimension-1;i++){
		if((x[i]<x[i+1] && y[i]<y[i+1]) || (x[i]>x[i+1] && y[i]>y[i+1])) splus++;
		if((x[i]<x[i+1] && y[i]>y[i+1]) || (x[i]>x[i+1] && y[i]<y[i+1])) sminus++;
	}
	return (splus-sminus)/((double)Dimension*((double)Dimension-1)/2);
}

double KendallDist(double *x, double *y, long Dimension){
	return 1-KendallCorrel(x, y, Dimension);
}

double SpearmanCorrel(double *x, double *y, long Dimension){
	double xpruh, ypruh,cit, jm1,jm2,xx[1000],yy[1000];
	long i,j;
	cit=0; jm1=0; jm2=0;xpruh=0;ypruh=0;
	for(i=0;i<Dimension;i++){
		xx[i]=1;
		yy[i]=1;
	}
	for(i=0;i<Dimension;i++){
		for(j=0;j<Dimension;j++){
			if(x[i]>x[j]) xx[i]++;
			if(y[i]>y[j]) yy[i]++;
		}
	}//then in xx and yy are corresponding ranks
	for(i=0;i<Dimension;i++){
		xpruh+=xx[i];
		ypruh+=yy[i];
	}
	xpruh/=Dimension;
	ypruh/=Dimension;
	for(i=0;i<Dimension;i++){
		cit+=(xx[i]-xpruh)*(yy[i]-ypruh);
		jm1+=Na2(xx[i]-xpruh);
		jm2+=Na2(yy[i]-ypruh);
	}
	return cit/(sqrt(jm1)*sqrt(jm2));
}

double SpearmanDist(double *x, double *y, long Dimension){
	return 1-SpearmanCorrel(x, y, Dimension);
}


double Ha2(double *x, double *y, long Dimension){
	double xy;
	long i;
	xy=0;
	for(i=0;i<Dimension;i++)
		xy+=1.0-1.0/(1.0+fabs(x[i]-y[i]));
	return xy/Dimension;
}

double Haha(double *x, double *y, long Dimension){
	double xy, eps,xy0;
	long i;
/*	eps=0.0001;
	xy=1;
	for(i=0;i<Dimension;i++)
		xy+=log(fabs(x[i]-y[i])+eps);//constant added not to cause zero result due to single case
//xy0=xy;
	xy/=(double)Dimension;
	xy=exp(xy);
        if(xy<=1.1*eps) xy=0.0;
//printf("Haha: %li %lg %lg %lg",Dimension, eps, xy, xy0);
//getchar();
*/
	xy0=0.0;
	xy=0.0;
	for(i=1;i<Dimension;i++){
		eps=fabs(x[i]-y[i]);
		if(eps>1.0e-10)	{
			xy+=log(eps);
			xy0=1.0;
		}
	}
	if(xy0<0.5) return 0.0;
	else {
		xy/=(double)Dimension;
		xy=exp(xy);
		return xy;
	}
}

double Ha22(double *x, double *y, long Dimension){
	double xy,z;
	long i;
	z=0;
	for(i=0;i<Dimension;i++){
		xy=fabs(x[i]-y[i]);
		z+=2.677*xy +0.319*xy*xy -0.538*xy*xy*xy;//5.06 -2.877 0.55//4.3 -2 0.3
	}
	return z/Dimension+Ha2(x,y,Dimension);
}

double distance(double *x, double *y, double *ZCovMat, long Dimension, 
		double Metrics, long Class) {
	double dist2; 
	long j;
	if(Metrics>0){//then Lp metrics
		dist2=0;
		for(j=0;j<Dimension;j++) dist2+=Na(fabs(x[j]-y[j]), Metrics);
//printf("%lg %lg %lg\n",x[j],y[j],dist2);
//getchar();
		return Na(dist2, 1/Metrics);
	}
	if((long)(Metrics-0.001)==-1){//then Mahalanobis independent of class
		return MahalanobisDist(x, y, ZCovMat, Dimension, -1);
	}	
	if((long)(Metrics-0.001)==-2){//then Mahalanobis dependent of class CDM
		return MahalanobisDist(x, y, ZCovMat, Dimension, Class);
	}	
        if((long)(Metrics-0.001)== -3){// Orloci metrics = sqrt(2*(1-<x,y>/(||x||*||y||)) \n");
		return Orloci(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -4){// Angular semimetrics = arccos(<x,y>/(||x||*||y||)) \n");
		return Angular(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -5){// Clark metrics = sqrt{1/nSUM[(xi-yi)/(|xi|+|yi|)]^2} \n");
		return Clark(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -6){// Lorentz metrics = SUM[ln(1+|xi-yi|)] \n");
		return Lorentz(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -7){// Canberra metrics = SUM[|xi-yi|/(|xi|+|yi|)] \n");
		return Canberra(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -8){// Bray-Curtis (nearly) metrics = SUM|xi-yi|/SUM(xi+yi) \n");
		return BrayCurtis(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -9){// Intersection (nearly) metrics = 1-SUM[min(xi,yi)]/min[sum xi,sum yi] \n");
		return Intersection(x, y, Dimension);
	}
        if((long)(Metrics-0.001)==-10){// Cayley-Klein-Hilbert metrics = arccosh{abs[1-<x,y>]/[sqrt abs(1-<x,x>)*sqrt abs(1-<y,y>)] \n");
		return CayleyKleinHilbert(x, y, Dimension);
	}
        if((long)(Metrics-0.001)==-11){// Weierstrass metrics = arccosh{abs[sqrt(1+<x,x>)*sqrt(1+<y,y>)-<x,y>]} \n");
		return Weierstrass(x, y, Dimension);
	}	
        if((long)(Metrics-0.001)==-12){// Hassanat metrics = 
                // = sum{if min xi,yi>0 then 1-(1+min xi,yi)/(1+max xi,yi) else 1-(1+min xi,yi + abs min xi,yi)/(1+max xi, yi +abs min xi,i)}
		return Hassanat(x, y, Dimension);
	}
	//added 17.12.2016:
	if((long)(Metrics-0.001)==-15){//h2 metric derived from Hassanat metric. 
		return Ha2(x,y,Dimension);
		}
	if((long)(Metrics-0.001)==-16){//h3 multiplicative distance. 
		return Haha(x,y,Dimension);
	}
	if((long)(Metrics-0.001)==-17){//Ha2 distance with squares instead of abs values. 
		return Ha22(x,y,Dimension);
	}

	//added 9.4.2016:
        if((long)(Metrics-0.001)== -21){// Pearson (nearly) metrics 
		return PearsonDist(x, y, Dimension);
	}
        if((long)(Metrics-0.001)== -22){// Jacknife (nearly) metrics 
		return JacknifeDist(x, y, Dimension);
	}
        if((long)(Metrics-0.001)==-23){// GoodmanKruskalDist (nearly) metrics
		return GoodmanKruskalDist(x, y, Dimension);
	}
        if((long)(Metrics-0.001)==-24){// Kendall metrics 
		return KendallDist(x, y, Dimension);
	}	
        if((long)(Metrics-0.001)==-25){// Spearman metrics 
		return SpearmanDist(x, y, Dimension);
	}
/**/
	//else  Euclidean metrics
	dist2=0;
	for(j=0;j<Dimension;j++) dist2+=Na(x[j]-y[j], 2.0);
	return Na(dist2, 1/2.0);
}

//added 23.1.2016
long FiboNo(long n, long *FiboNo2){
//Returns Fibonacci number f(n) and gives Fibonacci number for n-2, i.e. f(n-2), in FiboNo2[0].
//For n>45 returns f(45) and f(43), in FiboNo2[0].
	long i, x, f, ff;
	FiboNo2[0]=0;
	if(n==2 || n==3) FiboNo2[0]=1;
	if(n>45) {
		FiboNo2[0]=701408733;
		return 1836311903;
	}
	ff=1;
	f=1;
	for (i=1;i<n;i++){
		x=f+ff;
		f=ff;
		ff=x;
		if(i==n-3) FiboNo2[0]=ff; 
	}
	return ff;
}

double LinRegCountX(double *X, double *slope, double *C, long N, long ignore){
	//A standard linear regression of function count=f(X) giving slope and constant C.
	//returns sqrt of the mean residuum squared.
	//ignore "ignore" samples from the beginning, if appropriate.
	double Sri2, Sri, Si, Siri, ri, residu, Iapr, sumresidu2;
	long i, trida, NC;
	Sri2=0; Sri =0; Si  =0;  Siri=0; NC=0; trida=0;
	for(i=ignore;i<N;i++) {
		ri=X[i]; 
		Sri2=Sri2+ri*ri;
		Sri =Sri+ri;
		Si  =Si+i;
		Siri=Siri+i*ri;
		NC++;
	}//thus we have data for regression equation; N is the last point used
	slope[trida]=   (NC*Siri-Sri*Si)/(NC*Sri2-Sri*Sri);
	C[trida]=(Si*Sri2-Sri*Siri)/(NC*Sri2-Sri*Sri);//regression constant 
	//RESIDUALS evaluation
	sumresidu2=0;
	for(i=ignore;i<N;i++) {
		ri=X[i]; 
		Iapr=C[trida]+slope[trida]*ri;//linear approximation
		residu=(double)i-Iapr;//residuum 
		sumresidu2+=Na2(residu);
	}
	//printf("\n");
	return sqrt(sumresidu2/NC);
}



//this procedure added 28.1.2016 
double LinRegCount3med(double *X, double *slope, double *C, long N, long ignore){
	//A 3 medians linear regression of function count=f(X) giving slope and constant C.
	//returns sqrt of the mean residuum squared.
	//ignore "ignore" samples from the beginning, if appropriate.
	//Even it is not too valuable algorithm [1], [2], in any case it is  
	//relatively robust especially to outliers in the "ends" of data set
	//[1] Walters,E.J., Morell,C.H., Auer,R.E.: An Investigation of the Median-Median 
	//    Method of Linear regression. Journ. of Statistics Education Vol. 14, No. 2 (2006).
	//    Available at http://www.amstat.org/publications/jse/v14n2/morell.html .
	//[2] Jessica ViTran: Summary 3 3med methds.doc. (2011). Available at 
	//    https://further.wikispaces.com/file/detail/Summary%203%203med%20methds.doc .
	double dil, dil1,dil2,dil3, ri, residu, Iapr, sumresidu2, cmed1, cmed2; 
	double cmed3,x1,x2,x3, sumresidu;
	long i, trida, zb, NC;
	trida=0; dil1=0.0; dil2=0.0; dil3=0.0;
	//3 medians regression
	zb=(long)fmod(N-ignore,3);
	dil=(N-ignore)/3;
	if(zb==0) dil1=dil2=dil3=dil;
	if(zb==1) {dil1=dil3=dil; dil2=dil+1;}
	if(zb==2) {dil1=dil3=dil+1; dil2=dil;}
	cmed1=ignore+((double)dil1+1)*0.5;
	cmed2=ignore+dil1+((double)dil2+1)*0.5;
	cmed3=ignore+dil1+dil2+((double)dil3+1)*0.5;
	if(fmod(dil1,2)==1) x1=X[(long)(cmed1+0.01)];
	else x1=0.5*(X[(long)(cmed1)]+X[(long)(cmed1+1.0)]);
	if(fmod(dil2,2)==1) x2=X[(long)(cmed2+0.01)];
	else x2=0.5*(X[(long)(cmed2)]+X[(long)(cmed2+1.0)]);
	if(fmod(dil3,2)==1) x3=X[(long)(cmed3+0.01)];
	else x3=0.5*(X[(long)(cmed3)]+X[(long)(cmed3+1.0)]);
	slope[trida]=(cmed3-cmed1)/(x3-x1);
	C[trida]=(cmed1+cmed2+cmed3-slope[trida]*(x1+x2+x3))/3;//regression constant 
	//RESIDUALS evaluation
	sumresidu2=0;
	sumresidu=0;
	NC=0;
	for(i=ignore;i<N;i++) {
		ri=X[i]; 
		Iapr=C[trida]+slope[trida]*ri;//linear approximation
		residu=(double)i-Iapr;//residuum 
		sumresidu2+=Na2(residu);
		sumresidu+=residu;
		NC++;
	}
	C[0]+=sumresidu/NC;
	sumresidu=0;
	for(i=ignore;i<N;i++) {
		ri=X[i]; 
		Iapr=C[trida]+slope[trida]*ri;//linear approximation
		residu=(double)i-Iapr;//residuum 
		sumresidu2+=Na2(residu);
		sumresidu+=residu;
	}
	printf("%lg  %lg\n", slope[0],C[0]);
	return sqrt(sumresidu2/NC);
}


double Sagginess(double *X, double *slope, double *C, long N, long ignore){
	//evaluation of sagginess, (cz: "prohnuti", "poklesnut") when given a standard 
	//linear regression of function count=f(x) with help of slope and constant C.
	//An count (N minus ignore) is divided into three parts and this function
	//returns a difference between mean of end parts and the mean of the middle part.
	//In each part a mean value of residuum is computed.
	//The result is positive for a convex function and negative for a concave function.
	//ignore "ignore" samples from the beginning, if appropriate.
	double a,b,c, prohnuti, ri, Iapr, residu;
	long i, ca,cb,cc,NC;
	a=0;b=0;c=0;ca=0;cb=0;cc=0; NC=0;
	for(i=ignore;i<N;i++) {
		NC++;
		ri=X[i]; 
		Iapr=C[0]+slope[0]*ri;//linear approximation
		residu=(double)i-Iapr;//residuum 
		//printf("%lg\n",residu);
		if(i<(N+2*ignore)/3) {
			a+=residu;
			ca++;
		}
		if(i>=(N+2*ignore)/3 && i<(2*N+ignore)/3) {
			b+=residu;
			cb++;
		}
		if(i>=(2*N+ignore)/3) {
			c+=residu;
			cc++;
		}
	}
	prohnuti=b/cb-(a/ca+c/cc)/2;
	return prohnuti; 
}

double WhereMinFceFibo(double (*FUNKCE)(double q, double *par, long npar), 
					   double a, double b, long n, double *fcex, double *par, long npar){
	//returns a position of a minimum of an unimodal function in <a,b>
	//and gives a minimum value of a function in fcex[0]
	//using Fibonacci sections method.
	//n is order of the Fibonacci number.
	//The resulting position of function minima lies in an interval of length (b-a)/Fibonacci(n)
	// eg.  (b-a)/89 for n=10  
	//      (b-a)/144 for n=11
	//      (b-a)/987 for n=15  order ~10^-3
	//      (b-a)/1597 for n=16
	//      (b-a)/10946 for n=20  order ~10^-4
	//      (b-a)/1346269 for n=30  order ~10^-6
	//      (b-a)/102334155 for n=39  order ~10^-8
	//Do not use n>45; it is set to 45 for numerical reasons.
	//The function FUNKCE needs some parameters stored in array par[] of length npar; 
	//  here parameters are transferred only, not used in any way.
	long i,fin,fin2;
	double x,xstr,fxstr,fx;
	fin=FiboNo(n,&fin2); x=0.0; fx=0.0; 
	xstr=b-(b-a)*(double)fin2/((double)fin);
	fxstr=FUNKCE(xstr, par, npar);
	for(i=1;i<n;i++){
		x=a+b-xstr;
		fx=FUNKCE(x, par, npar);
		if(fx<fxstr) {//then x is closer to the solution than xstr; b can be moved to the left into xstr.
			if(x<xstr) b=xstr;
			else a=xstr;   //(xstr<x) a can be moved into  xstr; x remains the same
			xstr=x;
			fxstr=fx;
		}
		else {// is closer to the solution than x;  xstr, fxstr remains the same
			if(x<xstr) a=x;
			else b=x;
		}
	}
	fcex[0]=fx;
	return x;
}

//added 17.Feb.,2017:
long contrl(){
//A text file CNTRL is supposed with an integer on the first row. 
//If this file exists, the function returns that number.
//if there is no CNTRL file or there is no number, the function returns 0. 
	FILE* rr;
	double a[50000];
	long num, CRA;
	if ((rr = fopen("CNTRL", "rt")) == NULL) return 0;
	if((CRA=CtiRadek(rr, a)) > 0){ 
		num=(long)a[0];
		fclose(rr);
		return num;
	}
	if(CRA==0){
		fclose (rr);
		return CRA;
	}
	if(CRA<0){
		return CRA;
	}
	return CRA;
}


/*KONEC ======================================================*/
